import useCtrlActionsCtx from '@thatreact/ctrl/hooks/useCtrlActionsCtx'
import useCtrlCtx from '@thatreact/ctrl/hooks/useCtrlCtx'
import useCtrlState from '@thatreact/ctrl/hooks/useCtrlState'
import CtrlState from '@thatreact/ctrl/types/CtrlState'
import React, { MutableRefObject, useCallback, useContext, useMemo, useRef } from 'react'

export function JustAddedController({ children }) {
	// STATE
	const _s = useCtrlState<JustAddedData>({
		ids: [],
	})

	// ACTIONS
	const _a = useRef<JustAddedActions>({} as JustAddedActions)
	_a.current.doJustAdded = useCallback(
		(id: string, timeout: number = 5000) => {
			_s.current.ids.push(id)
			_s.current.setChanged()
			setTimeout(() => {
				_s.current.ids = _s.current.ids.filter(v => v !== id)
				_s.current.setChanged()
			}, timeout)
		},
		[_s]
	)

	// CTX
	const ctx = useCtrlCtx(_s)

	// ACTIONS CTX
	const actionsCtx = useCtrlActionsCtx(_a, 'JustAdded')

	return (
		<JustAddedContext.Provider value={ctx}>
			<JustAddedActionsContext.Provider value={actionsCtx}>
				{children}
			</JustAddedActionsContext.Provider>
		</JustAddedContext.Provider>
	)
}
export default JustAddedController

export type JustAddedData = {
	ids: string[]
}
export type JustAddedState = CtrlState<JustAddedData>
export const JustAddedContext = React.createContext<JustAddedData>(null)

export type JustAddedActions = {
	doJustAdded: (v: string, timeout?: number) => void
}
export type JustAddedActionsState = MutableRefObject<JustAddedActions>
export const JustAddedActionsContext = React.createContext<JustAddedActions>(null)

export const useJustAdded = () => {
	const { ids } = useContext(JustAddedContext)
	return useMemo(() => ids, [ids])
}

export const useIsJustAdded = (id: string) => {
	const { ids } = useContext(JustAddedContext)
	const isJustAdded = ids.indexOf(id) >= 0
	return useMemo(() => isJustAdded, [isJustAdded])
}

export const useJustAddedActions = () => useContext(JustAddedActionsContext)
